package com.ruoyi.business.service.impl;

import java.util.List;
import com.ruoyi.common.utils.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.business.mapper.OrderManageMapper;
import com.ruoyi.business.domain.OrderManage;
import com.ruoyi.business.service.IOrderManageService;
import com.ruoyi.common.core.text.Convert;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

/**
 * 【请填写功能名称】Service业务层处理
 * 
 * @author wms
 * @date 2022-10-10
 */
@Service
public class OrderManageServiceImpl extends ServiceImpl<OrderManageMapper, OrderManage> implements IOrderManageService
{
    @Autowired
    private OrderManageMapper orderManageMapper;

    /**
     * 查询【请填写功能名称】
     * 
     * @param orderId 【请填写功能名称】主键
     * @return 【请填写功能名称】
     */
    @Override
    public OrderManage selectOrderManageByOrderId(String orderId)
    {
        return orderManageMapper.selectOrderManageByOrderId(orderId);
    }

    /**
     * 查询【请填写功能名称】
     *
     * @param goodsId 【请填写功能名称】主键
     * @return 【请填写功能名称】
     */
    @Override
    public OrderManage selectNextOrder(String goodsId)
    {
        return orderManageMapper.selectNextOrder(goodsId);
    }

    /**
     * 查询【请填写功能名称】
     *
     * @param orderManage 【请填写功能名称】主键
     * @return 【请填写功能名称】
     */
    @Override
    public OrderManage selectPreOrder(OrderManage orderManage)
    {
        return orderManageMapper.selectPreOrder(orderManage);
    }

    /**
     * 查询【请填写功能名称】列表
     * 
     * @param orderManage 【请填写功能名称】
     * @return 【请填写功能名称】
     */
    @Override
    public List<OrderManage> selectOrderManageList(OrderManage orderManage)
    {
        return orderManageMapper.selectOrderManageList(orderManage);
    }

    /**
     * 查询待出工单列表，以goodsId分类
     *
     * @param orderManage 【请填写功能名称】
     * @return 【请填写功能名称】
     */
    @Override
    public List<OrderManage> selectOrderListForOut(OrderManage orderManage)
    {
        return orderManageMapper.selectOrderListForOut(orderManage);
    }

    /**
     * 新增【请填写功能名称】
     * 
     * @param orderManage 【请填写功能名称】
     * @return 结果
     */
    @Override
    public int insertOrderManage(OrderManage orderManage)
    {
        return orderManageMapper.insertOrderManage(orderManage);
    }

    /**
     * 修改【请填写功能名称】
     * 
     * @param orderManage 【请填写功能名称】
     * @return 结果
     */
    @Override
    public int updateOrderManage(OrderManage orderManage)
    {
        orderManage.setUpdateTime(DateUtils.getNowDate());
        return orderManageMapper.updateOrderManage(orderManage);
    }

    /**
     * 修改【请填写功能名称】
     *
     * @param orderManage 【请填写功能名称】
     * @return 结果
     */
    @Override
    public int upOneOrder(OrderManage orderManage)
    {
        return orderManageMapper.upOneOrder(orderManage);
    }

    /**
     * 工单取消
     *
     * @param orderIds 需要取消的工单号
     * @return 结果
     */
    @Override
    public int cancelOrdersByOrderIds(String orderIds)
    {
        return orderManageMapper.cancelOrdersByOrderIds(Convert.toStrArray(orderIds));
    }

    /**
     * 批量删除【请填写功能名称】
     * 
     * @param orderIds 需要删除的【请填写功能名称】主键
     * @return 结果
     */
    @Override
    public int deleteOrderManageByOrderIds(String orderIds)
    {
        return orderManageMapper.deleteOrderManageByOrderIds(Convert.toStrArray(orderIds));
    }

    /**
     * 删除【请填写功能名称】信息
     * 
     * @param orderId 【请填写功能名称】主键
     * @return 结果
     */
    @Override
    public int deleteOrderManageByOrderId(String orderId)
    {
        return orderManageMapper.deleteOrderManageByOrderId(orderId);
    }
}
